/// <reference types="node" />

import { Context, ILogger, Identity, ObjectId, Resolvable, TypeRegistry } from '@e2/core';
import { Index, MongoApplicationSettings, MongoClient, MongoCollection, MongoDatabase, MongoModel } from '@e2/mongodb';
import { InternalWebContext, WebApplication, WebApplicationSettings, WebContext, WebController } from '@e2/web';
import { Constructor } from 'agentframework';

export interface WebFoundationApplicationMetadata {
	version: string;
}
export interface IdentityProfile {
	_id: ObjectId | null;
	displayName: string;
	displayImageURL: string;
}
export interface IAuditable {
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt?: Date;
	updatedFrom?: string;
	updatedBy?: IdentityProfile;
}
export interface IExpirable {
	expires?: Date;
}
export interface IVertex extends MongoModel, IAuditable, IExpirable {
}
export interface IBlockable {
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
}
export interface IDeletable {
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface IExternalUserIdentity {
	type: string;
	key?: string;
	value: string;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
}
export interface IExternalUserOptionalFields {
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: ObjectId | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
}
export interface IExternalUser extends IVertex, IExternalUserMandatoryFields, IExternalUserOptionalFields, IExpirable, IAuditable, IBlockable, IDeletable {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: ObjectId | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export interface IActivatable {
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
}
export interface IProfile {
	displayName: string;
	displayImageURL: string;
}
export interface UserSettings {
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
}
export interface UserAddress {
	address: string;
	postalCode: string;
	city: string;
	state: string;
	countryCode: string;
	country: string;
}
export interface IUser extends IVertex, IActivatable, IBlockable, IDeletable, IProfile {
	displayName: string;
	displayImageURL: string;
	signature?: any;
	location?: any;
	address: UserAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: UserSettings;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface IAuthenticationProfile extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: ObjectId | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export interface IAuthenticationToken {
	token_type: string;
	scope: string;
	expires_in: number;
	ext_expires_in: number;
	access_token: string;
	refresh_token: string;
}
export interface IAuthenticator {
	getAuthorizeURL(redirect_uri: string, state: any): string;
	getToken(redirect_uri: string, code: string): Promise<IAuthenticationToken>;
	refreshToken(token: IAuthenticationToken): Promise<any>;
	getProfile(token: IAuthenticationToken): Promise<Partial<IAuthenticationProfile>>;
	getFailureURL(err: Error): string;
	getSuccessURL(data: any): string;
}
export declare const SYSTEM_MAX_UPLOAD_SIZE: number;
export declare const SYSTEM_MAX_NUMBER_OF_ACCOUNTS_PER_USER = 1;
export declare class WebFoundationApplicationSettings extends WebApplicationSettings implements MongoApplicationSettings {
	ADMIN_EMAIL: string;
	MONGODB_URL: string;
	MONGODB_DATABASE: string;
	MONGODB_EVENT_LOG_LEVEL: string;
	WEBAPP_SESSION_COOKIE_PATH: string;
	WEBAPP_SESSION_COOKIE_KEY: string;
	WEBAPP_SESSION_COOKIE_SECRET: Array<string>;
	WEBAPP_SESSION_COOKIE_SECURED: boolean;
	WEBAPP_SESSION_COOKIE_EXPIRES_IN_SECONDS: number;
	WEBAPP_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEBAPP_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEBAPP_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEBAPP_EMAIL_VERIFICATION_RESEND_COOLDOWN_IN_MILLISECONDS: number;
	WEBAPP_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEBAPP_LOGIN_PATH: string;
	WEBAPP_LOGIN_SUCCESS_PATH: string;
	WEBAPP_LOGIN_FAILED_PATH: string;
	WEBAPP_LOGOUT_PATH: string;
	WEBAPP_LOGOUT_COMPLETE_PATH: string;
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
	JDK_ACCESS_KEY: string;
	OAUTH_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OAUTH_REDIRECT_LIST: Array<string>;
	OAUTH_NO_REDIRECTS_LIST: Array<string>;
	OIDC_PROVIDERS: Array<any>;
	OAUTH_GOOGLE: boolean;
	OAUTH_GOOGLE_CLIENT_ID: string;
	OAUTH_GOOGLE_CLIENT_SECRET: string;
	OAUTH_GOOGLE_SCOPE: Array<string>;
	OAUTH_MICROSOFT: boolean;
	OAUTH_MICROSOFT_TENANT_ID: string;
	OAUTH_MICROSOFT_CLIENT_ID: string;
	OAUTH_MICROSOFT_CLIENT_SECRET: string;
	OAUTH_MICROSOFT_BASE_URL: string;
	OAUTH_MICROSOFT_SCOPE: Array<string>;
	OAUTH_GITLAB: boolean;
	OAUTH_GITLAB_CLIENT_ID: string;
	OAUTH_GITLAB_CLIENT_SECRET: string;
	OAUTH_GITLAB_BASE_URL: string;
	OAUTH_GITLAB_SCOPE: Array<string>;
	OAUTH_GITHUB: boolean;
	OAUTH_GITHUB_CLIENT_ID: string;
	OAUTH_GITHUB_CLIENT_SECRET: string;
	OAUTH_GITHUB_SCOPE: Array<string>;
	OAUTH_WEIBO: boolean;
	OAUTH_WEIBO_CLIENT_ID: string;
	OAUTH_WEIBO_CLIENT_SECRET: string;
	OAUTH_WEIBO_SCOPE: Array<string>;
	OAUTH_WECHAT: boolean;
	OAUTH_WECHAT_CLIENT_ID: string;
	OAUTH_WECHAT_CLIENT_SECRET: string;
	OAUTH_WECHAT_SCOPE: Array<string>;
	OAUTH_TELEGRAM: boolean;
	OAUTH_TELEGRAM_BOT_ID: string;
	OAUTH_TELEGRAM_BOT_SECRET: string;
	OAUTH_TELEGRAM_BOT_TOKEN: string;
	OAUTH_TELEGRAM_PAYMENT_TOKEN: string;
}
declare class SecuredCookie extends Resolvable<WebFoundationApplicationSettings> {
	private readonly crypts;
	private readonly base72;
	constructor(app: Context<WebFoundationApplicationSettings>);
	sign(data: string): string;
	verify(signed: string): string | false;
}
declare class SessionCookieManager extends Resolvable<WebFoundationApplicationSettings> {
	security: SecuredCookie;
	get(ctx: WebContext<WebFoundationApplicationSettings>): string | null;
	set(ctx: WebContext<WebFoundationApplicationSettings>, data: string): void;
	clear(ctx: WebContext<WebFoundationApplicationSettings>): void;
}
export interface Session {
	session: ObjectId | null;
	user: ObjectId | null;
	token: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	profile: IdentityProfile | null;
}
export declare class WebIdentityService extends Resolvable<WebFoundationApplicationSettings> {
	readonly name: string;
	sessionCookie: SessionCookieManager;
	create(ctx: WebContext<WebFoundationApplicationSettings>, id?: string): Partial<Session>;
	update(ctx: WebContext<WebFoundationApplicationSettings>, session: Partial<Session>): void;
	clear(ctx: WebContext<WebFoundationApplicationSettings>): void;
}
export declare class WebFoundationApplication<T extends WebFoundationApplicationSettings> extends WebApplication<T> {
	WebIdentityService: WebIdentityService;
	protected createRequestState(ctx: InternalWebContext<T>, state?: any): any;
	protected createRequestIdentity(ctx: InternalWebContext<T>, id: string, address: string): Identity;
	onInitApplication(): void;
}
declare class InternalSymbol {
	static readonly ServiceContext: unique symbol;
}
declare abstract class WebFoundationUserService<T extends WebFoundationApplicationSettings> extends Resolvable<T> {
	private [InternalSymbol.ServiceContext];
	constructor(app: WebFoundationApplication<T>, ctx: any);
	readonly domain: WebFoundationDomain<T>;
	addResponseHeader(name: string, value: number | string): void;
	getRequestHeader(name: string): string | string[] | undefined;
}
export interface FetchConfiguration {
	method: string;
	url: string;
	data: any;
}
export interface FetchResponse {
	ok: boolean | number;
	headers: any;
	status: number;
	error?: string;
	errcode?: number;
	message?: string;
	stack?: string;
	result?: any;
	results?: Array<any>;
}
declare class WebIdentity implements Identity, Session {
	id: string;
	address: string;
	session: ObjectId | null;
	user: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	token: ObjectId | null;
	profile: IdentityProfile | null;
}
export interface WebFoundationToken {
}
export interface WebFoundationDomain<T extends WebFoundationApplicationSettings> extends ILogger {
	readonly identity: WebIdentity;
	readonly token: WebFoundationToken;
	readonly settings: T;
	set(key: string, value: any): any;
	has(key: string): boolean;
	fetch(url: string, init?: FetchConfiguration): Promise<FetchResponse>;
	fetch(init: FetchConfiguration): Promise<FetchResponse>;
	construct<T>(type: Constructor, transit?: boolean): T;
	resolve<T>(type: Constructor, transit?: boolean): Promise<T>;
	collection<T>(type: Constructor): T;
}
export interface IEdge extends IAuditable, IExpirable {
	outType: string;
	label: string;
	inType: string;
	outV: string | any;
	inV: string | any;
	meta: any;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
declare class Collection<T extends MongoModel> extends MongoCollection<T> {
}
declare class EdgeModel extends MongoModel implements IEdge {
	outType: string;
	label: string;
	inType: string;
	outV: ObjectId | any;
	inV: ObjectId | any;
	meta: any;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	collection(): Collection<EdgeModel>;
}
export interface VertexModelConstructor<T extends VertexModel> {
	new (): T;
	getType(): string;
}
declare class VertexModel extends MongoModel implements IVertex {
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	collection(): Collection<VertexModel>;
	$edge(): string | false;
	addEdgeFrom(label: string, outModel: VertexModel, meta?: any): Promise<EdgeModel | null>;
	addEdgeTo(label: string, inModel: VertexModel, meta?: any): Promise<EdgeModel | null>;
	fetch(): Promise<VertexModel>;
	findEdges(): Promise<Array<EdgeModel>>;
	findOutEdges(): Promise<Array<EdgeModel>>;
	findInEdges(): Promise<Array<EdgeModel>>;
	findFromVertices<T extends VertexModel>(outCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findToVertices<T extends VertexModel>(inCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findOutVertices<T extends VertexModel>(outCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findInVertices<T extends VertexModel>(inCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findOutTypes(label: string): Promise<Array<string>>;
	findInTypes(label: string): Promise<Array<string>>;
	findOutLabels(inModelConstructor: VertexModelConstructor<VertexModel>): Promise<Array<string>>;
	findInLabels(inModelConstructor: VertexModelConstructor<VertexModel>): Promise<Array<string>>;
}
export declare class UserModel extends VertexModel implements IUser {
	displayName: string;
	displayImageURL: string;
	signature?: any;
	location?: any;
	address: UserAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: UserSettings;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface WebFoundationContextState {
	db: Graph;
	user?: UserModel;
	[x: string]: any;
}
export interface WebFoundationContext<T extends WebFoundationApplicationSettings> extends WebContext<T> {
	identity: WebIdentity;
	state: WebFoundationContextState;
	domain: WebFoundationDomain<T>;
}
export interface WfCtx extends WebFoundationContext<WebFoundationApplicationSettings> {
}
export interface WfApp extends WebFoundationApplication<WebFoundationApplicationSettings> {
}
export declare class Graph extends MongoDatabase {
	readonly identity: Partial<WebIdentity>;
	constructor(client: MongoClient, app: WfApp, ctx: WfCtx);
	audit(action: string, updates?: object, status?: boolean): object;
}
export declare class EmailModel extends VertexModel {
	name: string;
	displayName: string;
	user: ObjectId;
	verified: boolean;
	verifiedAt: Date;
	verifiedBy: IdentityProfile;
	verifiedFrom: string;
	verifyToken: string;
	verifyTokenFailures: number;
	verifyTokenExpires: Date;
	salt: string;
	password: string;
	resetPasswordAt: Date;
	resetPasswordToken: string;
	resetPasswordTokenExpires: Date;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class EmailCollection extends Collection<EmailModel> {
	constructor(db: Graph);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export declare class ExternalUserModel extends VertexModel implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'openid' | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string;
	domain: string;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string;
	displayImageFile: ObjectId;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class ExternalUserCollection extends Collection<ExternalUserModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class SessionModel extends VertexModel implements IBlockable, IDeletable {
	device: string;
	user: ObjectId;
	roles: Array<string>;
	account: string;
	token: ObjectId;
	tokens: Array<any>;
	previous?: ObjectId;
	impersonated: boolean;
	impersonatedAt: Date;
	impersonatedFrom: string;
	impersonatedBy: IdentityProfile;
	impersonatedReason: string;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class SessionCollection extends Collection<SessionModel> {
	constructor(db: Graph);
}
export declare class UserCollection extends Collection<UserModel> {
	constructor(db: Graph);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export declare class CurrentUser extends UserModel {
	static [TypeRegistry.Initializer](app: WebApplication<WebFoundationApplicationSettings>, ctx: WfCtx): Promise<UserModel | null> | null;
}
export interface WfDomain extends WebFoundationDomain<WebFoundationApplicationSettings> {
}
export declare class ExternalUserMutations {
	static FindExternalUser(domain: WfDomain, profile: IExternalUserKeyFields): Promise<ExternalUserModel | null>;
	static FindOrCreateExternalUser(domain: WfDomain, profile: Partial<IAuthenticationProfile>): Promise<ExternalUserModel>;
}
export declare class SessionMutations {
	static CreateSession(domain: WfDomain, user: UserModel, allows: Array<string>, context?: any, previousSession?: SessionModel | null | undefined, impersonated?: boolean | undefined): Promise<SessionModel>;
	static CreateOrUpdateSession(domain: WfDomain, user: UserModel, allows: Array<string>, account: string, context?: any): Promise<SessionModel | null>;
	static ExtendSession(domain: WfDomain, context: any): Promise<SessionModel>;
	static RestoreSession(domain: WfDomain, sessionToRestore: SessionModel, context: any, reason: string): Promise<SessionModel | null>;
	static TerminateSession(domain: WfDomain, reason: string): Promise<SessionModel | null>;
}
export declare class UserMutations {
	static FindOrCreateFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static CreateFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static FindOrCreateFromEmail(domain: WfDomain, email: EmailModel): Promise<UserModel>;
	static CreateFromEmail(domain: WfDomain, email: EmailModel): Promise<UserModel>;
	static Update(domain: WfDomain, user: UserModel, props: any): Promise<UserModel | null>;
}
export declare class ExternalUserGetters {
	static FindExternalUser(domain: WfDomain, profile: IExternalUserMandatoryFields): Promise<ExternalUserModel | null>;
}
export declare class SessionGetters {
	static GetUserById(domain: WfDomain, id: any): Promise<UserModel | null>;
	static GetExternalUserFromUser(domain: WfDomain, user: UserModel, provider: string, providerProtocol: string): Promise<ExternalUserModel | null>;
}
export declare class UserGetters {
	static GetUserById(domain: WfDomain, id: any): Promise<UserModel | null>;
	static GetExternalUserFromUser(domain: WfDomain, user: UserModel, provider: string, providerProtocol: string): Promise<ExternalUserModel | null>;
}
export interface RestOptions {
	headers?: any;
	query?: any;
	responseType?: string;
}
export declare class RestClient {
	private defaults;
	constructor(defaults?: RestOptions);
	private makeAxiosOptions;
	post(url: string, data: any, options?: RestOptions): Promise<any>;
	postBuffer(url: string, data: any, options?: RestOptions): Promise<Buffer>;
	put(url: string, data: any, options?: RestOptions): Promise<any>;
	putBuffer(url: string, data: any, options?: RestOptions): Promise<Buffer>;
	get(url: string, options?: RestOptions): Promise<any>;
	getBuffer(url: string, options?: RestOptions): Promise<Buffer>;
	delete(url: string, options?: RestOptions): Promise<any>;
	deleteBuffer(url: string, options?: RestOptions): Promise<Buffer>;
}
export interface MicrosoftUser {
	'@odata.context': string;
	id: string;
	businessPhones: Array<string>;
	displayName: string;
	givenName: string;
	jobTitle: string;
	mail: string;
	mobilePhone: string;
	officeLocation: string;
	preferredLanguage: string;
	surname: string;
	userPrincipalName: string;
}
export interface MicrosoftPhotoMetadata {
	'@odata.context': string;
	'@odata.mediaContentType': string;
	'@odata.mediaEtag': string;
	id: string;
	height: number;
	width: number;
}
export declare class MicrosoftClient extends RestClient {
	constructor(access_token: string);
	profile(): Promise<MicrosoftUser>;
	photo(): Promise<MicrosoftPhotoMetadata | null>;
	photoImage(photoSize: string): Promise<Buffer>;
	photoImageURL(): Promise<string | null>;
	users(): Promise<Array<any>>;
}
export interface FileResource {
	_id: ObjectId;
	id: string;
	token: string;
	url: string;
	path: string;
	name: string;
	type: string;
	size: number;
	md5: string;
	sha1: string;
	sha256: string;
	description: string;
	downloadQuota: number;
	download: number;
	totalViews: number;
	totalDownloads: number;
	notValidBefore: Date;
	notValidAfter: Date;
	createdAt: Date;
}
export interface FileHash {
	md5: string;
	sha1: string;
	sha256: string;
	ipfs: string;
}
export interface FileAccessToken {
	token: string;
	description?: string;
	notValidBefore: Date;
	notValidAfter?: Date;
	downloadQuota?: number;
	download: number;
}
export interface FileAccessStatistics {
	view: number;
	download: number;
}
export interface FileChunkInfo {
	_id: ObjectId;
	offset: number;
	length: number;
}
declare class FileModel extends VertexModel implements IExpirable, IAuditable {
	path: string;
	name: string;
	originalname: string;
	type: string;
	size: number;
	hash: FileHash;
	chunks: Array<FileChunkInfo>;
	context: any;
	stats: FileAccessStatistics;
	tokens: Array<FileAccessToken>;
	expires: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
}
declare class FileService extends Resolvable<WebFoundationApplicationSettings> {
	private jdk;
	enabled(): boolean;
	upload(filename: string, data: Buffer, type: string): Promise<Partial<FileResource> | null>;
	save(domain: WfDomain, data: Buffer, type: string, path: string, filename: string, overwrite?: boolean, originalname?: string): Promise<Partial<FileResource>>;
	metadata(domain: WfDomain, id: string): Promise<FileModel | null>;
	content(domain: WfDomain, id: string, token: string): Promise<FileModel | null>;
}
export interface MicrosoftAuthenticatorOptions {
	clientId: string;
	clientSecret: string;
	tenantId: string;
	scope: Array<string>;
}
export interface MicrosoftIdToken {
	ver: string;
	iss: string;
	sub: string;
	aud: string;
	exp: number;
	iat: number;
	nbf: number;
	name: string;
	preferred_username: string;
	oid: string;
	email: string;
	tid: string;
	aio: string;
}
export interface MicrosoftToken extends IAuthenticationToken {
	id_token: MicrosoftIdToken;
}
export declare class MicrosoftAuthenticator extends WebFoundationUserService<WebFoundationApplicationSettings> implements IAuthenticator {
	FileService: FileService;
	scope: string;
	oauth: any;
	certifiedEmails: string[];
	init(options: MicrosoftAuthenticatorOptions): void;
	getAuthorizeURL(redirect_uri: string, state: any): string;
	getSuccessURL(user: UserModel): string;
	getFailureURL(err: Error): string;
	getToken(redirect_uri: string, code: string): Promise<MicrosoftToken>;
	refreshToken(token: any): Promise<any>;
	getProfile(token: MicrosoftToken): Promise<Partial<IAuthenticationProfile>>;
}
export declare class SessionService extends Resolvable<WebFoundationApplicationSettings> {
	Terminate(domain: WfDomain, reason: string): Promise<SessionModel | null>;
	CreateNew(domain: WfDomain, user: UserModel, context: any, allows: Array<string>, previousSession?: SessionModel, impersonated?: boolean): Promise<SessionModel>;
	throwErrorForInvalidUser(user: UserModel): void;
	throwErrorForInvalidSession(session: SessionModel): void;
}
export declare class WebAuthenticationService extends Resolvable<WebFoundationApplicationSettings> {
	protected IdentityService: WebIdentityService;
	protected SessionService: SessionService;
	Login(ctx: WfCtx, user: UserModel, allows?: Array<string>): Promise<SessionModel>;
	private updateWebIdentity;
}
export declare class UserService extends Resolvable<WebFoundationApplicationSettings> {
	findById(domain: WfDomain, id: string | ObjectId | null | undefined): Promise<UserModel | null>;
	findByEmail(domain: WfDomain, emailAddress: string | null | undefined): Promise<UserModel | null>;
	CreateFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
}
export declare class ExternalUserService extends Resolvable<WebFoundationApplicationSettings> {
	findOrCreateExternalUser(domain: WfDomain, profile: Partial<IAuthenticationProfile>): Promise<ExternalUserModel>;
}
export declare abstract class WebFoundationController<T extends WebFoundationApplicationSettings> extends WebController<T> {
	protected getFailureURL(ctx: WfCtx, err: Error): Promise<string>;
}
declare abstract class OAuthController<T extends WebFoundationApplicationSettings> extends WebFoundationController<T> {
	protected ExternalUserService: ExternalUserService;
	protected UserService: UserService;
	abstract authenticator(ctx: WfCtx): Promise<IAuthenticator>;
	abstract callback(ctx: WfCtx, externalUser: ExternalUserModel, foundUsers: UserModel | null, loggedInUser?: UserModel | null): Promise<UserModel>;
	__OAuthController_Start(ctx: WfCtx): Promise<void>;
	__OAuthController_Callback(ctx: WfCtx): Promise<void>;
}
export declare abstract class MicrosoftOAuthController<T extends WebFoundationApplicationSettings> extends OAuthController<T> {
	abstract configure(ctx: WfCtx): Promise<MicrosoftAuthenticatorOptions>;
	authenticator(ctx: WfCtx): Promise<IAuthenticator>;
}